/***************************************************************************
 *   Copyright (C) 2009 by Andreas Lindenau   *
 *   DL4JAL@darc.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <QtGui>
#include <cmath>
#include <QtGlobal>
#include <QString>

#include "kschreiber.h"
#include "configfile.h"


kschreiber::kschreiber()
{
  version = "V1.07";
  verdatum = " vom 13.02.2014";

  ksettings.setminX(0.0);
  ksettings.setmaxX(10.0);
  ksettings.setnumXTicks(20);
  ksettings.setminY(0.0);
  ksettings.setmaxY(500.0);
  ksettings.setnumYTicks(20);

///////////////////////////////////////////////////////////////////////////////
//Schnittstelle zum NWT7 erzeugen erzeugen fuer alle Widgets
//  picmodem = new QextSerialPort("/dev/ttyS0");
//  delete picmodem;
  picmodem = new Modem();
//  picmodem->opentty(weichen.str_tty);
  QObject::connect( picmodem, SIGNAL(setTtyText(QString)), this, SLOT(setAText(QString)));
//  bttyOpen = picmodem->opentty(weichen.str_tty);
///////////////////////////////////////////////////////////////////////////////
  
  testtimer = new QTimer(this);
  QObject::connect(testtimer, SIGNAL(timeout()), this, SLOT(readbfm() ));
  imaxtimer = new QTimer(this);
  QObject::connect(imaxtimer, SIGNAL(timeout()), this, SLOT(readimax() ));
  akurventimer = new QTimer(this);
  QObject::connect(akurventimer, SIGNAL(timeout()), this, SLOT(readakurventimer() ));
  ekurventimer = new QTimer(this);
  QObject::connect(ekurventimer, SIGNAL(timeout()), this, SLOT(readekurventimer() ));

  awidget = new QTabWidget();
  QObject::connect( awidget, SIGNAL(currentChanged(int)), this, SLOT(settabwidget(int)));
  
//  awidget->setTabPosition(QTabWidget::West);
  setCentralWidget(awidget);
  wkschreiber = new QWidget();
  setMinimumSize (600, 480);

  awidget->addTab(wkschreiber, tr("Kennlinien","TabWindow"));
  grafik = new WDisplay(wkschreiber);
  
  gkurven = new QGroupBox(tr("Kurven"),wkschreiber);
  
  bkurvestart = new QPushButton(gkurven);
  bkurvestart->setText(tr("Start"));
  QObject::connect( bkurvestart, SIGNAL( clicked()), this, SLOT(clickkurvenstart()));
  
  bclearcurven = new QPushButton(gkurven);
  bclearcurven->setText(tr("Kurven loeschen"));
  QObject::connect( bclearcurven, SIGNAL( clicked()), this, SLOT(clickbclearcurven()));
  
  bsethintergrund = new QPushButton(gkurven);
  bsethintergrund->setText(tr("Kurven als Hintergrund"));
  QObject::connect( bsethintergrund, SIGNAL( clicked()), this, SLOT(clickbsethintergrund()));
  
  //labelkurvenanz = new QLabel(tr("Anzahl:","Label"), gkurven);
  //spkurvenanz = new QSpinBox(gkurven);
  //spkurvenanz->setRange(1,5);
  //spkurvenanz->setSingleStep(1);
  //spkurvenanz->setValue(1);
  
  labelui1 = new QLabel("", gkurven);
  boxui1 = new QComboBox(gkurven);
  QObject::connect( boxui1, SIGNAL(currentIndexChanged(int)), this, SLOT(aendernboxui1()));
  
  labelui2 = new QLabel("", gkurven);
  boxui2 = new QComboBox(gkurven);
  QObject::connect( boxui2, SIGNAL(currentIndexChanged(int)), this, SLOT(aendernboxui2(int)));
  
  textedit = new QTextEdit(wkschreiber);
  
  gibasis = new QGroupBox(wkschreiber);
  i10ua = new QRadioButton("10 uA", gibasis);
  QObject::connect(i10ua, SIGNAL(toggled(bool)), this, SLOT(aendernIb()));
  i100ua = new QRadioButton("100 uA", gibasis);
  QObject::connect(i100ua, SIGNAL(toggled(bool)), this, SLOT(aendernIb()));
  i1ma = new QRadioButton("1 mA", gibasis);
  QObject::connect(i1ma, SIGNAL(toggled(bool)), this, SLOT(aendernIb()));
  
  i10ua->setChecked(TRUE);

  checkclrcurve = new QCheckBox(tr("auto. Loeschen"), gkurven);
  
//#############################################################################
//Option Widget
//#############################################################################
  woption = new QWidget();
  awidget->addTab(woption, tr("Einstellungen / Messreihen","TabWindow"));
  
    
  gcolor = new QGroupBox(tr("Farbeinstellungen"),woption);
  bcolorhintergrund = new QPushButton(gcolor);
  bcolorhintergrund->setText(tr("Hintergrund"));
  QObject::connect( bcolorhintergrund, SIGNAL( clicked()), this, SLOT(clickcolorhintergrund()));
  bcolorschrift = new QPushButton(gcolor);
  bcolorschrift->setText(tr("Schrift/Linien"));
  QObject::connect( bcolorschrift, SIGNAL( clicked()), this, SLOT(clickcolorschrift()));
  bcolorueberschrift = new QPushButton(gcolor);
  bcolorueberschrift->setText(tr("Ueberschrift"));
  QObject::connect( bcolorueberschrift, SIGNAL( clicked()), this, SLOT(clickcolorueberschrift()));
  bcolorlinie = new QPushButton(gcolor);
  bcolorlinie->setText(tr("Kurven"));
  QObject::connect( bcolorlinie, SIGNAL( clicked()), this, SLOT(clickcolorlinie()));
  bcolorhlinie = new QPushButton(gcolor);
  bcolorhlinie->setText(tr("Hintergrund-Kurven"));
  QObject::connect( bcolorhlinie, SIGNAL( clicked()), this, SLOT(clickcolorhlinie()));
  
  gdatei = new QGroupBox(tr("Messreihen"),woption);
  edateiname = new QLineEdit("default.csv",gdatei);
  labeldateiname = new QLabel(tr("Dateiname","Label"), gdatei);
  sphnr = new QSpinBox(gdatei);
  sphnr->setRange(1,1000);
  sphnr->setValue(1);
  labelhnr = new QLabel(tr("ObjektNr:","Label"), gdatei);
  checkaufzeichnung = new QCheckBox(tr("Aufzeichnen"), gdatei);
  QObject::connect( checkaufzeichnung, SIGNAL( stateChanged ( int )), this, SLOT(clickaufzeichnung(int)));

  
//#############################################################################
//Test Widget
//#############################################################################
  wtest = new QWidget();
  awidget->addTab(wtest, tr("HW Test","TabWindow"));
  
  grelais = new QGroupBox(tr("Relaisansteuerung"),wtest);
  bucplus = new QPushButton(grelais);
  bucplus->setText(tr("Kollektor+ Basis+"));
  QObject::connect( bucplus, SIGNAL( clicked()), this, SLOT(clickucplus()));
  bucminus = new QPushButton(grelais);
  bucminus->setText(tr("Kollektor- Basis+"));
  QObject::connect( bucminus, SIGNAL( clicked()), this, SLOT(clickucminus()));
  bubplus = new QPushButton(grelais);
  bubplus->setText(tr("Kollektor+ Basis-"));
  QObject::connect( bubplus, SIGNAL( clicked()), this, SLOT(clickubplus()));
  bubminus = new QPushButton(grelais);
  bubminus->setText(tr("Kollektor- Basis-"));
  QObject::connect( bubminus, SIGNAL( clicked()), this, SLOT(clickubminus()));
  b10uA = new QPushButton(grelais);
  b10uA->setText(tr("Basisstrom 10uA"));
  QObject::connect( b10uA, SIGNAL( clicked()), this, SLOT(clickb10uA()));
  b100uA = new QPushButton(grelais);
  b100uA->setText(tr("Basisstrom 100uA"));
  QObject::connect( b100uA, SIGNAL( clicked()), this, SLOT(clickb100uA()));
  b1mA = new QPushButton(grelais);
  b1mA->setText(tr("Basisstrom 1mA"));
  QObject::connect( b1mA, SIGNAL( clicked()), this, SLOT(clickb1mA()));
  
  gspannungen = new QGroupBox(tr("Spannung"),wtest);
  buc0v = new QPushButton(gspannungen);
  buc0v->setText(tr("Kollektor 0 Volt"));
  QObject::connect( buc0v, SIGNAL( clicked()), this, SLOT(clickuc0v()));
  buc10v = new QPushButton(gspannungen);
  buc10v->setText(tr("Kollektor 10 Volt"));
  QObject::connect( buc10v, SIGNAL( clicked()), this, SLOT(clickuc10v()));
  bub0v = new QPushButton(gspannungen);
  bub0v->setText(tr("Basis 0 Volt"));
  QObject::connect( bub0v, SIGNAL( clicked()), this, SLOT(clickub0v()));
  bub10v = new QPushButton(gspannungen);
  bub10v->setText(tr("Basis 10 Volt"));
  QObject::connect( bub10v, SIGNAL( clicked()), this, SLOT(clickub10v()));
  
  ganzeige = new QGroupBox(tr("Spannungsanzeige"),wtest);
  lademitter = new QLabel(tr("A/D Emitter: ","Label"), ganzeige);
  ladbasis = new QLabel(tr("A/D Basis: ","Label"), ganzeige);
  ladcollector = new QLabel(tr("A/D Kollektor: ","Label"), ganzeige);
  liemitter = new QLabel(tr("Ic: ","Label"), ganzeige);
  lubasis = new QLabel(tr("Ub: ","Label"), ganzeige);
  lucollector = new QLabel(tr("Uc: ","Label"), ganzeige);;
  
  gstrommax = new QGroupBox(tr("Einstellung Imax"),wtest);
  spstrommax = new QSpinBox(gstrommax);
  spstrommax->setRange(5,500);
  spstrommax->setSingleStep(10);
  spstrommax->setValue(200);
  labelspstrommax = new QLabel(tr("(mA)","Label"), gstrommax);
  lstrommax = new QLabel(tr("Ubmax: 0,0 Volt","Label"), gstrommax);
  luc = new QLabel(tr("Uc: 0,0 Volt","Label"), gstrommax);
  bstrommax = new QPushButton(gstrommax);
  bstrommax->setText(tr("Ermitteln"));
  QObject::connect( bstrommax, SIGNAL( clicked()), this, SLOT(clickbstrommax()));
  
  gdatest = new QGroupBox(tr("D/A Dauertest"),wtest);
  bdastart = new QPushButton(gdatest);
  bdastart->setText(tr("Start"));
  QObject::connect( bdastart, SIGNAL( clicked()), this, SLOT(clickbdastart()));
  
  ucpol = 1.0;
  ubpol = 1.0;
//----------------------------------------------------------------  
  createActions();
  createMenus();
  createToolBars();
  createStatusBar();
  
  awidget->setEnabled(TRUE);
  kurvennummer = 0;
  
  repainttimer = new QTimer(this);
  QObject::connect(repainttimer, SIGNAL(timeout()), this, SLOT(actrepainttimer() ));
  openttytimer = new QTimer(this);
  QObject::connect(openttytimer, SIGNAL(timeout()), this, SLOT(fopentty() ));
  #ifdef Q_WS_WIN
    int i;
    QString qs;
    for(i=1;i<=20;i++){
      qs.sprintf("COM%i",i);
      listtty.append(qs);
    }
  #else
    listtty.append("/dev/ttyS0");
    listtty.append("/dev/ttyS1");
    listtty.append("/dev/ttyS2");
    listtty.append("/dev/ttyS3");
    listtty.append("/dev/ttyUSB0");
    listtty.append("/dev/ttyUSB1");
    listtty.append("/dev/ttyUSB2");
    listtty.append("/dev/ttyUSB3");
  #endif  
  idtty = 0; //mit Index 0 beginnen
  bvantwort = FALSE;
  bksgefunden = FALSE;
  readSettings();
  grafiksetzen();
  awidget->setEnabled(FALSE);
  repainttimer->start(1);
  bwstart = FALSE; 
  kurvenende = false;
}

void kschreiber::rs232einstellen()
{
  bool ok;
  
//QString getText ( QWidget * parent, const QString & title, const QString & label, QLineEdit::EchoMode mode = QLineEdit::Normal, const QString & text = QString(), bool * ok = 0, Qt::WindowFlags f = 0 )
  srs232 = QInputDialog::getText(this, tr("RS232","in InputBox"), tr("Einstellen der seriellen Schnittstelle"),
                                 QLineEdit::Normal,
                                 srs232, &ok);
  if(ok){
    picmodem->closetty();
    rs232isopen = picmodem->opentty(srs232);
    if(!rs232isopen){
      QMessageBox::about(this, tr("RS232 einstellen"),tr("Die RS232 ist nicht offen!"));
    }
  }
}

void kschreiber::setAText(QString stty){
  QString qs;
  qs = tr("Kennlinien-Schreiber");
  QString fwversion = tr("HW nicht gefunden");
  if(kversion >= 100)fwversion.sprintf("FW:%i.%02i",kversion / 100, kversion - 100);
  QString astty = qs + " - " + version + " - " + stty + " - " + fwversion;
  setWindowTitle(astty);
}

void kschreiber::closeEvent(QCloseEvent *event)
{
  cfgspeichern();
  writeSettings();
  if(checkaufzeichnung->isChecked()){
    clickaufzeichnung(Qt::Unchecked); 
  }
  event->accept();
}

void kschreiber::about()
{
  QString qs = tr("Diese Applikation dient der "
                  "Darstellung von Halbleiter "
                  "Kennlinien.<br><br> ")
                  +version+verdatum+"<br> (c) DL4JAL";

  QMessageBox::about(this, tr("Kennlinien-Schreiber"), qs);
}

void kschreiber::createActions()
{
  exitAct = new QAction(QIcon(":/beenden.xpm"),tr("&Beenden"), this);
  exitAct->setStatusTip(tr("Programm beenden"));
  connect(exitAct, SIGNAL(triggered()), this, SLOT(close()));

  rs232Act = new QAction(tr("&RS232 einstellen"), this);
  rs232Act->setStatusTip(tr("Serielle Schnittstelle einstellen"));
  connect(rs232Act, SIGNAL(triggered()), this, SLOT(rs232einstellen()));

  aboutAct = new QAction(tr("&Ueber"), this);
  aboutAct->setStatusTip(tr("Ueber die Applikation"));
  connect(aboutAct, SIGNAL(triggered()), this, SLOT(about()));

  aboutQtAct = new QAction(tr("Ueber &Qt"), this);
  aboutQtAct->setStatusTip("Show the Qt library's About box");
  connect(aboutQtAct, SIGNAL(triggered()), qApp, SLOT(aboutQt()));
  
  kennlausgangAct = new QAction(QIcon(":/kaus.xpm"),tr("&Ausgangskennlinie"), this);
  kennlausgangAct->setStatusTip(tr("Ausgangskennlinie darstellen"));
  connect(kennlausgangAct, SIGNAL(triggered()), this, SLOT(kennlausgang()));

  kennleingangAct = new QAction(QIcon(":/kein.xpm"),tr("&Eingangskennlinie"), this);
  kennleingangAct->setStatusTip(tr("Eingangskennlinie darstellen"));
  connect(kennleingangAct, SIGNAL(triggered()), this, SLOT(kennleingang()));

  npnAct = new QAction(QIcon(":/knpn.xpm"),tr("&NPN Transistor"), this);
  npnAct->setStatusTip(tr("Einstellung auf NPN Transistoren"));
  connect(npnAct, SIGNAL(triggered()), this, SLOT(setnpn()));
  
  pnpAct = new QAction(QIcon(":/kpnp.xpm"),tr("&PNP Transistor"), this);
  pnpAct->setStatusTip(tr("Einstellung auf PNP Transistoren"));
  connect(pnpAct, SIGNAL(triggered()), this, SLOT(setpnp()));
  
  nmosfetAct = new QAction(QIcon(":/knmos.xpm"),tr("N &Mosfet"), this);
  nmosfetAct->setStatusTip(tr("Einstellung auf N Mosfet"));
  connect(nmosfetAct, SIGNAL(triggered()), this, SLOT(setnmosfet()));
  
  pmosfetAct = new QAction(QIcon(":/kpmos.xpm"),tr("P M&osfet"), this);
  pmosfetAct->setStatusTip(tr("Einstellung auf P Mosfet"));
  connect(pmosfetAct, SIGNAL(triggered()), this, SLOT(setpmosfet()));
  
  njfetAct = new QAction(QIcon(":/knfet.xpm"),tr("N &JFet"), this);
  njfetAct->setStatusTip(tr("Einstellung auf N JFet"));
  connect(njfetAct, SIGNAL(triggered()), this, SLOT(setnjfet()));
  
  pjfetAct = new QAction(QIcon(":/kpfet.xpm"),tr("P J&Fet"), this);
  pjfetAct->setStatusTip(tr("Einstellung auf P JFet"));
  connect(pjfetAct, SIGNAL(triggered()), this, SLOT(setpjfet()));

  diodeAct = new QAction(QIcon(":/kdiode.xpm"),tr("&Diode"), this);
  diodeAct->setStatusTip(tr("Einstellung auf Dioden"));
  connect(diodeAct, SIGNAL(triggered()), this, SLOT(setdiode()));

  savecsvAct = new QAction(QIcon(":/filesavecsv.xpm"),tr("&Save Kurve als CSV Datei"), this);
  savecsvAct->setStatusTip(tr("Kurve speichern als CSV Datei"));
  connect(savecsvAct, SIGNAL(triggered()), this, SLOT(clicksavecsv()));
}

void kschreiber::createMenus()
{
  fileMenu = menuBar()->addMenu(tr("&Datei"));
  fileMenu->addAction(savecsvAct);
  fileMenu->addSeparator();
  fileMenu->addAction(exitAct);
  
  optionMenu = menuBar()->addMenu(tr("&Einstellungen"));
  optionMenu->addAction(rs232Act);

  kschreiberMenu = menuBar()->addMenu(tr("&Kennlinien"));
  kschreiberMenu->addAction(kennlausgangAct);
  kschreiberMenu->addAction(kennleingangAct);

  transistortypMenu = menuBar()->addMenu(tr("&Transistortyp"));
  transistortypMenu->addAction(npnAct);
  transistortypMenu->addAction(pnpAct);
  transistortypMenu->addAction(nmosfetAct);
  transistortypMenu->addAction(pmosfetAct);
  transistortypMenu->addAction(njfetAct);
  transistortypMenu->addAction(pjfetAct);
  transistortypMenu->addAction(diodeAct);
  
  menuBar()->addSeparator();

  helpMenu = menuBar()->addMenu(tr("&Hilfe"));
  helpMenu->addAction(aboutAct);
  helpMenu->addAction(aboutQtAct);
}

void kschreiber::createToolBars()
{
  fileToolBar = addToolBar(tr("Datei"));
  fileToolBar->addAction(exitAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(kennleingangAct);
  fileToolBar->addAction(kennlausgangAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(npnAct);
  fileToolBar->addAction(pnpAct);
  fileToolBar->addAction(nmosfetAct);
  fileToolBar->addAction(pmosfetAct);
  fileToolBar->addAction(njfetAct);
  fileToolBar->addAction(pjfetAct);
  fileToolBar->addAction(diodeAct);
  fileToolBar->addSeparator();
  fileToolBar->addAction(savecsvAct);
}

void kschreiber::kennlausgang(){
  getUI();
  if(kurventyp == eeingang){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  kurventyp = eausgang;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::kennleingang(){
  getUI();
  if(kurventyp == eausgang){
    grafik->loeschenAlleKurven();
    grafik->loeschenAlleHKurven();
  }
  kurventyp = eeingang;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setnpn(){
  getUI();
  halbleitertyp = enpn;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setpnp(){
  getUI();
  halbleitertyp = epnp;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setnmosfet(){
  getUI();
  halbleitertyp = enmosfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setpmosfet(){
  getUI();
  halbleitertyp = epmosfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setnjfet(){
  getUI();
  halbleitertyp = enjfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setpjfet(){
  getUI();
  halbleitertyp = epjfet;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setdiode(){
  getUI();
  halbleitertyp = ediode;
  setKurvenSchreiber();
  setUI();
}

void kschreiber::setKurvenSchreiber(){
  int p=0;
  QString qs;
  
  switch(kurventyp){
    case eeingang:
      switch(halbleitertyp){
        case enpn:
          gibasis->show();
          gibasis->setEnabled(TRUE);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Ibe (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.setzeinheit("Uce (V)");
          ksettings.setueberschrift(tr("NPN-Eingangskennlinienfeld Ic = f (Ibe)"));
          gkurven->setTitle(tr("Kurve-NPN-Eingang"));
          labelui1->setText("Uce (V):");
          labelui2->setText("Icmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epnp:
          gibasis->show();
          gibasis->setEnabled(TRUE);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Ibe (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.setzeinheit("Uce (V)");
          ksettings.setueberschrift(tr("PNP-Eingangskennlinienfeld Ic = f (Ibe)"));
          gkurven->setTitle(tr("Kurven-PNP-Eingang"));
          labelui1->setText("Uce (V):");
          labelui2->setText("Icmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("N Mosfet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-N Mosfet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("P Mosfet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-P Mosfet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enjfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(-10.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("N JFet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-N JFet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = -1.0;
          p = 2;
        break;
        case epjfet:
          gibasis->hide();
          ksettings.setxeinheit("Ugs (V)");
          ksettings.setminX(-10.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Uds (V)");
          ksettings.setueberschrift(tr("P JFet-Eingangskennlinienfeld Id = f (Ugs)"));
          gkurven->setTitle(tr("Kurven-P JFet-Eingang"));
          labelui1->setText("Uds (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = 1.0;
          p = 1;
        break;
        case ediode:
          gibasis->show();
          gibasis->setEnabled(TRUE);
          gibasis->setTitle(tr("I-Diode"));
          ksettings.setxeinheit("I-Diode (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Uac (V)");
          ksettings.setzeinheit("");
          ksettings.setueberschrift(tr("Diodenkennlinienfeld Uac = f (I-Diode)"));
          gkurven->setTitle(tr("Kurve-Diode"));
          labelui1->setText("Uac (V):");
          labelui2->setText("");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
      }
      break;
    case eausgang:
      switch(halbleitertyp){
        case enpn:
          gibasis->show();
          gibasis->setEnabled(TRUE);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Uce (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.setzeinheit("Ibe (uA)");
          ksettings.setueberschrift(tr("NPN-Ausgangskennlinienfeld Ic = f (Uce)"));
          gkurven->setTitle(tr("Kurven-NPN-Ausgang"));
          labelui1->setText("Ib (uA):");
          labelui2->setText("Icmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epnp:
          gibasis->show();
          gibasis->setEnabled(TRUE);
          gibasis->setTitle(tr("I-Basis"));
          ksettings.setxeinheit("Uce (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ice (mA)");
          ksettings.setzeinheit("Ibe (uA)");
          ksettings.setueberschrift(tr("PNP-Ausgangskennlinienfeld Ic = f (Uce)"));
          gkurven->setTitle(tr("Kurven-PNP-Ausgang"));
          labelui1->setText("Ib (uA):");
          labelui2->setText("Icmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("N Mosfet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-N Mosfet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
        case epmosfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("P Mosfet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-P Mosfet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = -1.0;
          p = 3;
        break;
        case enjfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("N JFet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-N JFet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = 1.0;
          ubpol = -1.0;
          p = 2;
        break;
        case epjfet:
          gibasis->hide();
          ksettings.setxeinheit("Uds (V)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(10.0);
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Ids (mA)");
          ksettings.setzeinheit("Ugs (V)");
          ksettings.setueberschrift(tr("P JFet-Ausgangskennlinienfeld Id = f (Uds)"));
          gkurven->setTitle(tr("Kurven-P JFet-Ausgang"));
          labelui1->setText("Ugs (V):");
          labelui2->setText("Idmax (mA):");
          ucpol = -1.0;
          ubpol = 1.0;
          p = 1;
        break;
        case ediode:
          gibasis->show();
          gibasis->setEnabled(TRUE);
          gibasis->setTitle(tr("I-Diode"));
          ksettings.setxeinheit("I-Diode (uA)");
          ksettings.setminX(0.0);
          ksettings.setmaxX(getIb());
          ksettings.setnumXTicks(20);
          ksettings.setyeinheit("Uac (V)");
          ksettings.setzeinheit("");
          ksettings.setueberschrift(tr("Diodenkennlinienfeld Uac = f (I-Diode)"));
          gkurven->setTitle(tr("Kurve-Diode"));
          labelui1->setText("Uac (V):");
          labelui2->setText("");
          ucpol = 1.0;
          ubpol = 1.0;
          p = 0;
        break;
      }
    break;
  }
  textedit->clear();
  int index = boxui2->currentIndex();
  aendernboxui2(index);
  grafik->setPlotSettings(ksettings);
  boxui1fuellen();
  boxui2fuellen();
  //qDebug(qPrintable(srs232));
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    qs.sprintf("@p%1i",p);
    //qDebug(qPrintable(qs));
    picmodem->writeLine(qs);
  }else{
    //warmstart();
  }
  grafiksetzen();
}

void kschreiber::createStatusBar()
{
  statusBar()->showMessage(tr("Ready"));
}

void kschreiber::readSettings()
{
  int i;
  QSettings settings("AFU", "kschreiber");
  QPoint pos = settings.value("pos", QPoint(200, 200)).toPoint();
  QSize size = settings.value("size", QSize(400, 400)).toSize();
  move(pos);
  resize(size);
  #ifdef Q_WS_WIN
  srs232 = settings.value("rs232", QString("COM1")).toString();
  #else
  srs232 = settings.value("rs232", QString("/dev/ttyS0")).toString();
  #endif
  
  picmodem->closetty();
  if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    defaultlesentty();
    picmodem->writeLine("@v");
    bvantwort = TRUE;
  }else{
    //warmstart();
  }
  openttytimer->start(200); //des Geraetes
  i = settings.value("kurventyp", int(eeingang)).toInt();
  kurventyp = (ekurventyp)i;
  i = settings.value("halbleitertyp", int(enmosfet)).toInt();
  halbleitertyp = (ehalbleitertyp)i;
  cfgladen();
  setKurvenSchreiber();
  setUI();
}

void kschreiber::writeSettings()
{
  QSettings settings("AFU", "kschreiber");
  settings.setValue("pos", pos());
  settings.setValue("size", size());
  settings.setValue("rs232", srs232);
  settings.setValue("kurventyp", int(kurventyp));
  settings.setValue("halbleitertyp", int(halbleitertyp));
  getUI();
  cfgspeichern();
}


kschreiber::~kschreiber()
{
}

void kschreiber::defaultlesentty(){
  char abuffer[256];

  if (rs232isopen){
    picmodem->readttybuffer(abuffer, 256);
  }
}

void kschreiber::clickucplus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p0");
    ucpol = 1.0;
    ubpol = 1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickucminus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p1");
    ucpol = -1.0;
    ubpol = 1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickubplus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p2");
    ucpol = 1.0;
    ubpol = -1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickubminus(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@p3");
    ucpol = -1.0;
    ubpol = -1.0;
  }else{
    warmstart();
  }
}

void kschreiber::clickb10uA(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@a0");
  }else{
    warmstart();
  }
}

void kschreiber::clickb100uA(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@a1");
  }else{
    warmstart();
  }
}

void kschreiber::clickb1mA(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@a2");
  }else{
    warmstart();
  }
}

void kschreiber::clickuc0v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@c0000");
  }else{
    warmstart();
  }
}

void kschreiber::clickuc10v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@c1023");
  }else{
    warmstart();
  }
}

void kschreiber::clickub0v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@b0000");
  }else{
    warmstart();
  }
}

void kschreiber::clickub10v(){
  //if(!rs232isopen)rs232isopen = picmodem->opentty(srs232);
  if(rs232isopen){
    picmodem->writeLine("@b1023");
  }else{
    warmstart();
  }
}

void kschreiber::clicksavecsv(){
  QFile f;
  QString qs, qs1, qs2, qs3, sdatei;
  unsigned char c1, c2;
  int messint, i, j, k, l=0;
  const double uconst = 10.0/1023.0;
  const double uconstjfet = 10.0/1023.0;
  const double iconst = 500.0/1023.0;
  double wert1, wert2;
  int readmax=0;

  //Festlegen der Datenmenge die maximal zurueck kommt
  switch(halbleitertyp){
  case enpn:
  case epnp:
  case enmosfet:
  case epmosfet:
  case ediode:
    readmax = 2050;
    break;
  case enjfet:
  case epjfet:
    readmax = 4098;
    break;
  }
  if(kurvenende){
    qs2 = boxui1->currentText();
    qs3 = boxui2->currentText();
    switch(kurventyp){
    case eeingang:
      switch(halbleitertyp){
      case enpn:
        qs = "# NPN Eingangskennlinie Ic[mA]=f(Ib) Uc[V]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Ib[uA];Ic[mA]";
        break;
      case epnp:
        qs = "# PNP Eingangskennlinie Ic[mA]=f(Ib) Uc[V]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Ib[uA];Ic[mA]";
        break;
      case enmosfet:
        qs = "# N-Mosfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case epmosfet:
        qs = "# P-Mosfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case enjfet:
        qs = "# N-Jfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case epjfet:
        qs = "# P-Jfet Eingangskennlinie Ids[mA]=f(Ugs) Uds[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Ugs[V];Ids[mA]";
        break;
      case ediode:
        qs = "# Diode Testkennlinie Uac[V]=f(Diode I)";
        qs1 = "Idiode[uA];Uac[V]";
        break;
      }
      break;
    case eausgang:
      switch(halbleitertyp){
      case enpn:
        qs = "# NPN Ausgangskennlinie Ic[mA]=f(Uce) Ib[uA]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Uce[V];Ic[mA]";
        break;
      case epnp:
        qs = "# PNP Ausgangskennlinie Ic[mA]=f(Uce) Ib[uA]=" + qs2 + " Icmax[mA]=" + qs3;
        qs1 = "Uce[V];Ic[mA]";
        break;
      case enmosfet:
        qs = "# N-Mosfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case epmosfet:
        qs = "# P-Mosfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case enjfet:
        qs = "# N-Jfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case epjfet:
        qs = "# P-Jfet Ausgangskennlinie Ids[mA]=f(Uds) Ugs[V]=" + qs2 + " Idmax[mA]=" + qs3;
        qs1 = "Uds[V];Ids[mA]";
        break;
      case ediode:
        qs = "# Diode Testkennlinie Uac[V]=f(Diode I)";
        qs1 = "Idiode[uA];Uac[V]";
        break;
      }
      break;
    }

    sdatei = QFileDialog::getSaveFileName(
          this
          ,tr("Kurve als CSV Speichern","FileDialog")
          ,"kurve.csv"
          ,tr("CSV Datei (*.csv *.csv);;""Alle Dateien (*.*)","FileDialog")
          );
    if(!sdatei.isEmpty()){
      f.setFileName(sdatei);
      if(f.open(QIODevice::WriteOnly)){
        QTextStream ds(&f);
        ds << qs << "\r\n";
        ds << qs1 << "\r\n";
        for(i=0; i<readmax; i+=2){
          c1 = rxbuffer[i];
          c2 = rxbuffer[i+1];
          messint = int(c1 + c2 * 256);
          //qDebug("messint:%i",messint);
          if(messint == 0x8000){
            kurvenende = TRUE;
            l = i - 2;
          }
          if(i == readmax){
            kurvenende = TRUE;
            l = i - 2;
          }
        }
        k=0;
        qDebug("readmax %i",readmax);
        qDebug("wert l %i",l);
        for(j=0; j<l; j+=2){
          c1 = rxbuffer[j];
          c2 = rxbuffer[j+1];
          messint = int(c1 + c2 * 256);
          //qDebug("messint %i",messint);
          switch(kurventyp){
          case eeingang:
            switch(halbleitertyp){
            case enpn:
            case epnp:
              wert1= double(k)*getIbconst();
              wert2= double(messint)*iconst;
              break;
            case ediode:
              wert1= double(k)*getIbconst();
              wert2= double(messint)*uconst;
              //messkurve.append(QPointF(double(k)*getIbconst(),double(messint)*iconst));
              break;
            case enmosfet:
            case epmosfet:
              wert1= double(k)*uconst;
              wert2= double(messint)*iconst;
              //messkurve.append(QPointF(double(k)*uconst,double(messint)*iconst));
              break;
            case enjfet:
            case epjfet:
              wert1= (double(k)*uconstjfet)-10.0;
              wert2= double(messint)*iconst;
              //messkurve.append(QPointF((double(k)*uconstjfet)-10.0,double(messint)*iconst));
              break;
            }
            break;
          case eausgang:
            switch(halbleitertyp){
            case enpn:
            case epnp:
            case enmosfet:
            case epmosfet:
            case enjfet:
            case epjfet:
              wert1= double(k)*uconst;
              wert2= double(messint)*iconst;
              //messkurve.append(QPointF(double(k)*uconst,double(messint)*iconst));
              break;
            case ediode:
              wert1= double(k)*getIbconst();
              wert2= double(messint)*uconst;
              //messkurve.append(QPointF(double(k)*getIbconst(),double(messint)*uconst));
              break;
            }

            break;
          }
          qs.sprintf("%2.4f;%2.4f",wert1,wert2);
          qs.replace('.',',');
          ds << qs << "\r\n";
          k++;
        }
        f.close();
      }
    }
  }else{
    QMessageBox::about(this, tr("CSV Datei speichern"),tr("Es wurde noch keine Kurve erzeugt!"));
  }
}

void kschreiber::settabwidget(int a){
  switch(a){
    case 0:
      testtimer->stop();
      imaxtimer->stop();
      defaultlesentty();
    break;
    case 1:
      defaultlesentty();
      testtimer->stop();
      akurventimer->stop();
      ekurventimer->stop();
    break;
    case 2:
      defaultlesentty();
      adindex = 0;
      if(rs232isopen){
        picmodem->writeLine("@m0");
      }else{
        warmstart();
      }
      akurventimer->stop();
      ekurventimer->stop();
      testtimer->start(800);
      rxindex = 0;
    break;
  }
}

void kschreiber::readbfm(){
  char abuffer[2];
  unsigned char c1, c2;
  int bad;
  QString qs;
  double ieconst = (1.0/1023.0)*500.0;
  double uconst = (1.0/1023.0)*10.0;
  int gelesen;
  
  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, 2);
    if(gelesen > 0){
      for(int i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
      testtimer->start(800);
    }
    if(rxindex >= 2){
      syncounter = 0;
      rxindex = 0;
      c1 = rxbuffer[0];
      c2 = rxbuffer[1];
      bad = int(c1 + c2 * 256);
      switch(adindex){
      case 0:
        qs.sprintf(" %i",bad);
        qs = tr("A/D Basis:") + qs;
        ladbasis->setText(qs);
        qs.sprintf("Ub: %02.2f V", uconst * double(bad) * ubpol);
        lubasis->setText(qs);
        break;
      case 1:
        qs.sprintf(" %i",bad);
        qs = tr("A/D Emitter:") + qs;
        lademitter->setText(qs);
        qs.sprintf("Ic: %03.1f mA", ieconst * double(bad) * ucpol);
        liemitter->setText(qs);
        break;
      case 2:
        qs.sprintf(" %i",bad);
        qs = tr("A/D Kollektor:") + qs;
        ladcollector->setText(qs);
        qs.sprintf("Uc: %02.2f V", uconst * double(bad) * ucpol);
        lucollector->setText(qs);
        break;
      }
      adindex++;
      if(adindex==3)adindex=0;
      picmodem->writeChar('@');
      switch(adindex){
      case 0:
        picmodem->writeLine("m0");
        break;
      case 1:
        picmodem->writeLine("m1");
        break;
      case 2:
        picmodem->writeLine("m2");
        break;
      }
      defaultlesentty();
      testtimer->start(800);
    }else{
      adindex=0;
      picmodem->writeLine("@m0");
      syncounter++;
      if(syncounter == 5){
        warmstart();
        awidget->setCurrentIndex(0);
      }
    }
  }else{
    warmstart();
  }
}

void kschreiber::clickbstrommax(){
  int intimax;
  const double ieconst = (1.0/1023.0)*500.0;
  QString qs;
  
  if(rs232isopen){
    picmodem->writeLine("@c1023");
  }else{
    warmstart();
  }
  double imax = spstrommax->value();
  intimax = abs(imax / ieconst);
  //qDebug("intimax %i",intimax);
  qs.sprintf("@t%04i",intimax);
  if(rs232isopen){
    picmodem->writeLine(qs);
  }else{
    warmstart();
  }
  defaultlesentty();
  testtimer->stop();
  rxindex = 0;
  imaxtimer->start(500);
}

void kschreiber::readimax(){
  char abuffer[4];
  unsigned char c1, c2;
  int bad1, bad2;
  double ubasis;
  QString qs;
  const double uconst = 10.0/1023.0;
  int gelesen;

  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, 4);
    if(gelesen > 0){
      for(int i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
    }
    if(rxindex >= 4){
      rxindex = 0;
      c1 = rxbuffer[0];
      c2 = rxbuffer[1];
      bad1 = int(c1 + c2 * 256);
      c1 = rxbuffer[2];
      c2 = rxbuffer[3];
      bad2 = int(c1 + c2 * 256);
      ubasis = bad1 * uconst;
      qs.sprintf("Ubmax: %2.3f Volt",ubasis);
      if(ubasis > 10.0)qs=tr("Eingaenge offen!!!");
      lstrommax->setText(qs);
      ubasis = bad2 * uconst;
      qs.sprintf("Uc: %2.3f Volt",ubasis);
      luc->setText(qs);
      defaultlesentty();
      picmodem->writeLine("@c0000");
      testtimer->start(800);
    }
  }else{
    warmstart();
  }
}

void kschreiber::resizeEvent( QResizeEvent * ){
  //nach Groessenaenderung grafische Elemente anpassen
  resize(this->rect().size());
  grafiksetzen();
  //qDebug("breite %i hoehe %i", gsize.width(), gsize.height());
}

void kschreiber::repaintEvent( QPaintEvent * ){
  //nach Groessenaenderung grafische Elemente anpassen
  //qDebug("breite %i hoehe %i", width(), height());
  grafiksetzen();
  QWidget::repaint();
}

void kschreiber::grafiksetzen(){
//  QSize gsize = wkschreiber->size();
//  grafik->resize(gsize);
//  grafik->setParameterPixmap();

  int w = wkschreiber->width();
  int h = wkschreiber->height();
  int gl = w-200;
  int a = 20;
  int b = 20;
  int f = 0;
  
  grafik->setGeometry(0,0,gl,h);
  
  gkurven->setGeometry(gl+10,0,180,150);
  
  bkurvestart->setGeometry(10, b+f*a, 150, 20);f++;
  bclearcurven->setGeometry(10, b+f*a, 150, 20);f++;
  bsethintergrund->setGeometry(10, b+f*a, 150, 20);f++;
  //labelkurvenanz->setGeometry(10, b+f*a, 100, 20); 
  //spkurvenanz->setGeometry(110, b+f*a, 50, 20);f++; 
  labelui1->setGeometry(10, b+f*a, 80, 20); 
  boxui1->setGeometry(90, b+f*a, 70, 20);f++;
  labelui2->setGeometry(10, b+f*a, 80, 20); 
  boxui2->setGeometry(90, b+f*a, 70, 20);f++;
  checkclrcurve->setGeometry(10, b+f*a, 150, 20); 
  
  gibasis->setGeometry(gl+10,160,180,90);
  i10ua->setGeometry(10,20,160,20);
  i100ua->setGeometry(10,40,160,20);
  i1ma->setGeometry(10,60,160,20);
  
  switch(halbleitertyp){
    case enpn:
    case epnp:
    case ediode:
      textedit->setGeometry(gl+10,260,180,h-260-10);
    break;
    default:
      textedit->setGeometry(gl+10,160,180,h-160-10);
    break;
  }
  
  grelais->setGeometry(10,10,180,190);
  bucplus->setGeometry(10,30,150,20);
  bucminus->setGeometry(10,50,150,20);
  bubplus->setGeometry(10,70,150,20);
  bubminus->setGeometry(10,90,150,20);
  b10uA->setGeometry(10,110,150,20);
  b100uA->setGeometry(10,130,150,20);
  b1mA->setGeometry(10,150,150,20);
  
  gspannungen->setGeometry(10,210,180,130);
  buc0v->setGeometry(10,30,150,20);
  buc10v->setGeometry(10,50,150,20);
  bub0v->setGeometry(10,70,150,20);
  bub10v->setGeometry(10,90,150,20);
  
  ganzeige->setGeometry(220,10,150,170);
  lademitter->setGeometry(10,30,120,20);
  ladbasis->setGeometry(10,50,120,20);
  ladcollector->setGeometry(10,70,120,20);
  liemitter->setGeometry(10,100,120,20);
  lubasis->setGeometry(10,120,120,20);
  lucollector->setGeometry(10,140,120,20);
  
  gstrommax->setGeometry(220,190,150,130);
  spstrommax->setGeometry(10,30,50,20);
  labelspstrommax->setGeometry(70,30,80,20);
  lstrommax->setGeometry(10,50,130,20);
  luc->setGeometry(10,70,130,20);
  bstrommax->setGeometry(10,90,100,20);
  
  gdatest->setGeometry(220,330,150,60);
  bdastart->setGeometry(10,30,100,20);
  
  gcolor->setGeometry(10,10,250,150);
  bcolorhintergrund->setGeometry(10,20,150,20);
  bcolorschrift->setGeometry(10,40,150,20);
  bcolorueberschrift->setGeometry(10,60,150,20);
  bcolorlinie->setGeometry(10,80,150,20);
  bcolorhlinie->setGeometry(10,100,150,20);
  
  gdatei->setGeometry(380,10,250,150);
  edateiname->setGeometry(100,20,140,20);
  labeldateiname->setGeometry(10,20,80,20);
  sphnr->setGeometry(70,50,60,20);
  labelhnr->setGeometry(10,50,40,20);
  checkaufzeichnung->setGeometry(140,50,100,20);
  
//  wkschreiber->resize(size());
//  grafik->setPlotSettings();
}
void kschreiber::clickkurvenstart(){
  const double uconst = 10.0/1023.0;
  const double iconst = 500.0/1023.0;
  double ub;
  double uc;
  double icmax;
  QString qs;
  
  for(int i=0; i<4098; i++) rxbuffer[i] = char(0);
  messkurve.clear();
  if(checkclrcurve->isChecked())clickbclearcurven();
  switch(kurventyp){
    case eausgang:
      switch(halbleitertyp){
        case enpn:
        case epnp:
          if(i10ua->isChecked())qs="@a0";
          if(i100ua->isChecked())qs="@a1";
          if(i1ma->isChecked())qs="@a2";
          //Ib-Bereich umschalten
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //qDebug(qPrintable(qs));
          //Wert im Bereich einstellen
          ub = getui1();
          ub = ub  / getIbconst();
          qs.sprintf("@b%04.0f",ub);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
//max Ic feststellen
          icmax = getui2() / iconst;
          qs.sprintf("@k%04.0f",icmax);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //Timer starten zum lesen der Daten
          akurventimer->start(100);
          idlezaehler = 0;
          //RS232 Empfangsbuffer leeren
          defaultlesentty();
          rxindex = 0;
        break;
        case epmosfet:
        case enmosfet:
          //Immer Ib max 1mA einstellen
          if(rs232isopen){
            picmodem->writeLine("@a2");
          }else{
            warmstart();
          }
          //Box ui1 auslesen
          ub = getui1() / uconst;
          qs.sprintf("@b%04.0f",ub);
          //qDebug(qPrintable(qs));
          //Gatespannung einstellen
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //Box ui2 auslesen
          icmax = getui2() / iconst;
          qs.sprintf("@k%04.0f",icmax);
          //qDebug(qPrintable(qs));
          //Id max einstellen
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          akurventimer->start(100);
          idlezaehler = 0;
          defaultlesentty();
          rxindex = 0;
        break;
        case enjfet:
          //Immer Ib max 1mA einstellen
          if(rs232isopen){
            picmodem->writeLine("@a2");
          }else{
            warmstart();
          }
          //Box ui1 auslesen
          ub = getui1() / uconst;
          if(ub <= 0.0)qs = "@p2";
          if(ub > 0.0)qs = "@p0";
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          ub = abs(ub);
          qs.sprintf("@b%04.0f",ub);
          //qDebug(qPrintable(qs));
          //Gatespannung einstellen
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //Box ui2 auslesen
          icmax = getui2() / iconst;
          qs.sprintf("@k%04.0f",icmax);
          //qDebug(qPrintable(qs));
          //Id max einstellen
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          akurventimer->start(100);
          idlezaehler = 0;
          defaultlesentty();
          rxindex = 0;
        break;
        case epjfet:
          //Immer Ib max 1mA einstellen
          if(rs232isopen){
            picmodem->writeLine("@a2");
          }else{
            warmstart();
          }
          //Box ui1 auslesen
          ub = getui1() / uconst;
          if(ub <= 0.0)qs = "@p3";
          if(ub > 0.0)qs = "@p1";
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          ub = abs(ub);
          qs.sprintf("@b%04.0f",ub);
          //qDebug(qPrintable(qs));
          //Gatespannung einstellen
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //Box ui2 auslesen
          icmax = getui2() / iconst;
          qs.sprintf("@k%04.0f",icmax);
          //qDebug(qPrintable(qs));
          //Id max einstellen
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          akurventimer->start(100);
          idlezaehler = 0;
          defaultlesentty();
          rxindex = 0;
        break;
        case ediode:
          if(i10ua->isChecked())qs="@a0";
          if(i100ua->isChecked())qs="@a1";
          if(i1ma->isChecked())qs="@a2";
          //Ib-Bereich umschalten
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine("@g");
          }else{
            warmstart();
          }
          //Timer starten zum lesen der Daten
          akurventimer->start(100);
          idlezaehler = 0;
          //RS232 Empfangsbuffer leeren
          defaultlesentty();
          rxindex = 0;
        break;
      }
    break;
    case eeingang:
      switch(halbleitertyp){
        case enpn:
        case epnp:
          //Basisstrom einstellen
          if(i10ua->isChecked())qs="@a0";
          if(i100ua->isChecked())qs="@a1";
          if(i1ma->isChecked())qs="@a2";
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          uc = getui1();
          uc = uc / uconst;
          qs.sprintf("@c%04.0f",uc);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          icmax = getui2() / iconst;
          qs.sprintf("@e%04.0f",icmax);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          ekurventimer->start(100);
          idlezaehler = 0;
          defaultlesentty();
          rxindex = 0;
        break;
        case epmosfet:
        case enmosfet:
          if(rs232isopen){
            picmodem->writeLine("@a2");
          }else{
            warmstart();
          }
          uc = getui1() / uconst;
          qs.sprintf("@c%04.0f",uc);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          icmax = getui2() / iconst;
          qs.sprintf("@e%04.0f",icmax);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          ekurventimer->start(100);
          idlezaehler = 0;
          defaultlesentty();
          rxindex = 0;
        break;
        case enjfet:
        case epjfet:
          if(rs232isopen){
            picmodem->writeLine("@a2");
          }else{
            warmstart();
          }
          uc = getui1() / uconst;
          qs.sprintf("@c%04.0f",uc);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          icmax = getui2() / iconst;
          qs.sprintf("@f%04.0f",icmax);
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          ekurventimer->start(100);
          idlezaehler = 0;
          defaultlesentty();
          rxindex = 0;
        break;
        case ediode:
          if(i10ua->isChecked())qs="@a0";
          if(i100ua->isChecked())qs="@a1";
          if(i1ma->isChecked())qs="@a2";
          //Ib-Bereich umschalten
          if(rs232isopen){
            picmodem->writeLine(qs);
          }else{
            warmstart();
          }
          //qDebug(qPrintable(qs));
          if(rs232isopen){
            picmodem->writeLine("@g");
          }else{
            warmstart();
          }
          //Timer starten zum lesen der Daten
          akurventimer->start(100);
          idlezaehler = 0;
          //RS232 Empfangsbuffer leeren
          defaultlesentty();
          rxindex = 0;
        break;
      }
    break;
  }
}

void kschreiber::readakurventimer(){
  int gelesen = 0;
  char abuffer[2050];
  unsigned char c1, c2;
  int messint=0, i, j, k, l=0;
//  bool kurvenende = FALSE;
  const double uconst = 10.0/1023.0;
  const double iconst = 500.0/1023.0;
  double ub;
  QString qs;
  
  kurvenende = false;
  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, 2050);
    if(gelesen > 0){
      for(i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
    }else{
      if(idlezaehler > 10){
        warmstart();
        return;
      }else{
        idlezaehler++;
      }
    }
    for(i=0; i<2050; i+=2){
      c1 = rxbuffer[i];
      c2 = rxbuffer[i+1];
      if(int(c1 + c2 * 256) == 0x8000){
        kurvenende = TRUE;
        l = i - 2;
      }
      if(i == 2050){
        kurvenende = TRUE;
        l = i - 2;
      }
    }
    if(kurvenende){
      //Ucollector Udrain auf 0 Volt eintstellen
      picmodem->writeLine("@c0000");
      akurventimer->stop();
      picmodem->writeLine("@b0000");
      k=0;
      for(j=0; j<l; j+=2){
        c1 = rxbuffer[j];
        c2 = rxbuffer[j+1];
        messint = int(c1 + c2 * 256);
        switch(halbleitertyp){
          case enpn:
          case epnp:
          case enmosfet:
          case epmosfet:
          case enjfet:
          case epjfet:
            messkurve.append(QPointF(double(k)*uconst,double(messint)*iconst));
          break;
          case ediode:
            messkurve.append(QPointF(double(k)*getIbconst(),double(messint)*uconst));
          break;
        }
        k++;
      }
      textedit->clear();
      //switch(halbleitertyp){
      //}
      ub = getui1();
      qs.sprintf("%1.2f",ub);
      grafik->setCurveData(kurvennummer, messkurve, qs);
      kurvennummer++;
      if(checkaufzeichnung->isChecked()){
        qs.sprintf("%i;%f;%f",sphnr->value(),ub,double(messint)*iconst);
        qs.replace('.',',');
        textedit->append(qs);
        messreihe << qs;
        int idx = sphnr->value();
        idx++;
        sphnr->setValue(idx);
      }
    }else{
      akurventimer->start(100);
    }
  }else{
    warmstart();
  }
}

void kschreiber::readekurventimer(){
  int gelesen = 0; //Anzahl der gelesenen Byte
  char abuffer[4098]; //temp lesebuffer RS232
  unsigned char c1, c2;
  int messint, i, j, k, l=0;
//  bool kurvenende = FALSE;
  const double uconst = 10.0/1023.0;
  const double uconstjfet = 10.0/1023.0;
  const double iconst = 500.0/1023.0;
  double uc;
  double icdelta, icdelta1, icdelta2, icdelta3;
  double ugdelta1=0.0, ugdelta2=0.0;
  double st;
  double uth = 0.0;
  QString qs;
  QString strich = "-------------------------";
  int readmax=0;
  
  //Festlegen der Datenmenge die maximal zurueck kommt
  kurvenende = false;
  switch(halbleitertyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case ediode:
      readmax = 2050;
    break;
    case enjfet:
    case epjfet:
      readmax = 4098;
    break;
  }
  if(rs232isopen){
    gelesen = picmodem->readttybuffer(abuffer, readmax);
    if(gelesen > 0){
      for(i=0; i < gelesen; i++){
        rxbuffer[rxindex] = abuffer[i];
        rxindex++;
      }
    }else{
      if(idlezaehler > 10){
        warmstart();
        return;
      }else{
        idlezaehler++;
      }
    }
    //qDebug("ki %i",rxindex);
    for(i=0; i<readmax; i+=2){
      c1 = rxbuffer[i];
      c2 = rxbuffer[i+1];
      if(int(c1 + c2 * 256) == 0x8000){
        kurvenende = TRUE;
        l = i - 2;
      }
      if(i == readmax){
        kurvenende = TRUE;
        l = i - 2;
      }
    }
    if(kurvenende){
      //Ucollector Udrain auf 0 Volt eintstellen
      picmodem->writeLine("@c0000");
      c1 = rxbuffer[l-2];
      c2 = rxbuffer[l-1];
      messint = int(c1 + c2 * 256);
      //icdelta2 ist letzter Ic Wert 
      icdelta2 = double(messint) * iconst;
      //icdelta3 ist Ic halbe
      icdelta3 = icdelta2 / 2.0;
      switch(halbleitertyp){
        case epnp:
        case enpn:
        case ediode:
          ugdelta2 = double(l / 2) * getIbconst();
        break;
        case enmosfet:
        case epmosfet:
          ugdelta2 = double(l / 2) * uconst;
        break;
        case enjfet:
        case epjfet:
          ugdelta2 = (double(l / 2) * uconstjfet) - 10.0;
        break;
      }
      icdelta1 = 0.0;
      //qDebug("l %i",l);
      k=0;
      for(j=0; j<l; j+=2){
        c1 = rxbuffer[j];
        c2 = rxbuffer[j+1];
        messint = int(c1 + c2 * 256);
        if((uth == 0.0) and (messint > 0)){
          switch(halbleitertyp){
            case epnp:
            case enpn:
            case ediode:
            break;
            case enmosfet:
            case epmosfet:
              uth = double(k) * uconst;
            break;
            case enjfet:
            case epjfet:
              uth = (double(k) * uconstjfet) - 10.0;
            break;
          }
        }
        icdelta = double(messint) * iconst;
        if(icdelta3 < icdelta){
          if(icdelta1 == 0.0){
            icdelta1 = icdelta;
            switch(halbleitertyp){
              case epnp:
              case enpn:
              case ediode:
                ugdelta1 = double(k) * getIbconst();
              break;
              case enmosfet:
              case epmosfet:
                ugdelta1 = double(k) * uconst;
              break;
              case enjfet:
              case epjfet:
                ugdelta1 = (double(k) * uconstjfet) - 10.0;
              break;
            }
          }
        }
        //qDebug("messint %i",messint);
        switch(halbleitertyp){
          case enpn:
          case epnp:
          case ediode:
            messkurve.append(QPointF(double(k)*getIbconst(),double(messint)*iconst));
          break;
          case enmosfet:
          case epmosfet:
            messkurve.append(QPointF(double(k)*uconst,double(messint)*iconst));
          break;
          case enjfet:
          case epjfet:
            messkurve.append(QPointF((double(k)*uconstjfet)-10.0,double(messint)*iconst));
          break;
        }
        k++;
      }
      textedit->clear();
      switch(halbleitertyp){
        case enpn:
        case epnp:
          qs.sprintf("Ib1: %2.3f uA",ugdelta1);
          textedit->append(qs);
          qs.sprintf("Ic1: %2.3f mA",icdelta1);
          textedit->append(qs);
          st = icdelta1 / (ugdelta1 / 1000.0);
          qs.sprintf("hfe1: %3.0f", st);
          textedit->append(qs);
          textedit->append(strich);
          qs.sprintf("Ib2: %2.3f uA",ugdelta2);
          textedit->append(qs);
          qs.sprintf("Ic2: %2.3f mA",icdelta2);
          textedit->append(qs);
          st = icdelta2 / (ugdelta2 / 1000.0);
          qs.sprintf("hfe2: %3.0f", st);
          textedit->append(qs);
          textedit->append(strich);
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          qs.sprintf("Uth: %2.3f V",uth);
          textedit->append(qs);
          textedit->append(strich);
          qs.sprintf("Ic1: %2.3f mA",icdelta1);
          textedit->append(qs);
          qs.sprintf("Ic2: %2.3f mA",icdelta2);
          textedit->append(qs);
          qs.sprintf("Ug1: %2.3f V",ugdelta1);
          textedit->append(qs);
          qs.sprintf("Ug2: %2.3f V",ugdelta2);
          textedit->append(qs);
          st = ((icdelta2 - icdelta1)/1000.0) / (ugdelta2 - ugdelta1);
          if(st < 1.0){
            qs.sprintf(" %3.0f mS",st*1000.0);
            qs = tr("Steilheit:") + qs;
          }else{
            qs.sprintf(" %2.3f S",st);
            qs = tr("Steilheit:") + qs;
          }
          textedit->append(qs);
          textedit->append(strich);
        break;
        case ediode:
        break;
      }
      if(checkaufzeichnung->isChecked()){
        switch(halbleitertyp){
          case enpn:
          case epnp:
            qs.sprintf("%i;%f;%f;%f;%f;%f;%f",sphnr->value()
                                             ,ugdelta1,icdelta1,icdelta1 / (ugdelta1 / 1000.0)
                                             ,ugdelta2,icdelta2,icdelta2 / (ugdelta2 / 1000.0));
          break;
          case enmosfet:
          case epmosfet:
          case enjfet:
          case epjfet:
            qs.sprintf("%i;%f;%f;%f;%f;%f;%f",sphnr->value()
                                          ,uth,ugdelta1,icdelta1,ugdelta2,icdelta2
                                          ,((icdelta2 - icdelta1)/1000.0) / (ugdelta2 - ugdelta1));
          break;
          case ediode:
          break;
        }
        qs.replace('.',',');
        textedit->append(qs);
        messreihe << qs;
        int idx = sphnr->value();
        idx++;
        sphnr->setValue(idx);
      }
      uc = getui1();
      QString qs;
      qs.sprintf("%1.2f",uc);
      grafik->setCurveData(kurvennummer, messkurve, qs);
      ekurventimer->stop();
      kurvennummer++;
    }else{
      ekurventimer->start(100);
    }
  }else{
    warmstart();
  }
}

void kschreiber::actrepainttimer(){
  grafiksetzen();
  resize(this->rect().size());
  repainttimer->stop();
  //qDebug("kschreiber::actrepainttimer()");
}

void kschreiber::clickbclearcurven(){
  grafik->loeschenAlleKurven();
  kurvennummer = 0;
}

void kschreiber::clickbsethintergrund(){
  grafik->Kurven2Hintergrund();
  grafik->loeschenAlleKurven();
  kurvennummer = 0;
}

void kschreiber::clickbdastart(){
  if(rs232isopen){
    picmodem->writeLine("@d");
  }else{
    warmstart();
  }
}

void kschreiber::setPolaritaet(){
  int p;
  QString qs;
  
  if((ubpol == 1.0) and (ucpol == 1.0))  p = 0;
  if((ubpol == 1.0) and (ucpol == -1.0)) p = 1;
  if((ubpol == -1.0) and (ucpol == 1.0)) p = 2;
  if((ubpol == -1.0) and (ucpol == -1.0))p = 3;
  if(rs232isopen){
    qs.sprintf("@p%1i",p);
    picmodem->writeLine(qs); //Polaritaeten Ub Uc einstellen
  }else{
    warmstart();
  }
}

void kschreiber::clickaufzeichnung(int status){
  QFile fmess;
  QString qs;
  QString s,s1;
  

  if(status == Qt::Checked){
    messreihe.clear();
    if(kurventyp == eausgang){
      switch(halbleitertyp){
        case enpn:
        case epnp:
          messreihe << "ObjNr;Ib;Ic";
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          messreihe << "ObjNr;Ug;Id";
        break;
        case ediode:
        break;
      }
    }else{
      switch(halbleitertyp){
        case enpn:
        case epnp:
          messreihe << "ObjNr;Ib1;Ic1;hfe1;Ib2;Ic2;hfe2";
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          messreihe << "ObjNr;Uth;Ug1;Ic1;Ug2;Ic2;S";
        break;
        case ediode:
        break;
      }
    }
    //qDebug("Start Aufzeichnung");
  }
  if(status == Qt::Unchecked){
    s = edateiname->text();
    if(!s.contains(".")) s += ".csv";
    fmess.setFileName(s);
    while(fmess.exists()){
      s1 += "$";
      fmess.setFileName(s+s1);
    }
    if(fmess.open(QIODevice::WriteOnly)){
      QTextStream ts(&fmess);
      for(int i=0; i<messreihe.size();i++){
        ts << messreihe.at(i) << endl;
      }
    }
    fmess.close();
    //qDebug("Stop Aufzeichnung");
  }
}

void kschreiber::aendernboxui1(){
  bool ok;
  
  switch(halbleitertyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case enjfet:
    case epjfet:
    break;
    case ediode:
      QString qs = boxui1->currentText();
      double wert = qs.toDouble(&ok);
      ksettings.setminY(0.0);
      ksettings.setmaxY(wert);
      ksettings.setnumYTicks(20);
      grafik->setPlotSettings(ksettings);
    break;
  }
}

double kschreiber::getui1const(){
  double wert = getui1()/1023.0;
  return wert;
}

double kschreiber::getui1(){
  bool ok;
  
  int index = boxui1->currentIndex();
  QString qs = boxui1->itemText(index);
  double wert = qs.toDouble(&ok);
  if(ok){
    return wert;
  }else{
    return 0.0;
  }
}

void kschreiber::aendernboxui2(int index){
  bool ok;
  QString qs;
  double wert;
  
  switch(halbleitertyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case enjfet:
    case epjfet:
      qs = boxui2->itemText(index);
      wert = qs.toDouble(&ok);
      ksettings.setminY(0.0);
      ksettings.setmaxY(wert);
      ksettings.setnumYTicks(20);
      grafik->setPlotSettings(ksettings);
    break;
    case ediode:
    break;
  }
}

double kschreiber::getui2const(){
  double wert = getui2()/1023.0;
  return wert;
}

double kschreiber::getui2(){
  bool ok;
  
  int index = boxui2->currentIndex();
  QString qs = boxui2->itemText(index);
  double wert = qs.toDouble(&ok);
  if(ok){
    return wert;
  }else{
    return 0.0;
  }
}

void kschreiber::aendernIb(){
  double wert;
  
  switch(kurventyp){
    case eeingang:
      switch(halbleitertyp){
        case enpn:
        case epnp:
        case ediode:
          if(i10ua->isChecked()){
            wert=10.0;
          }
          if(i100ua->isChecked()){
            wert=100.0;
          }
          if(i1ma->isChecked()){
            wert=1000.0;
          }
          ksettings.setminX(0.0);
          ksettings.setmaxX(wert);
          ksettings.setnumXTicks(20);
          grafik->setPlotSettings(ksettings);
        break;
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
        break;
      }
    break;
    case eausgang:
      switch(halbleitertyp){
        case ediode:
          if(i10ua->isChecked()){
            wert=10.0;
          }
          if(i100ua->isChecked()){
            wert=100.0;
          }
          if(i1ma->isChecked()){
            wert=1000.0;
          }
          ksettings.setminX(0.0);
          ksettings.setmaxX(wert);
          ksettings.setnumXTicks(20);
          grafik->setPlotSettings(ksettings);
        break;
        case enpn:
        case epnp:
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          boxui1fuellen();
        break;
      }
    break;
  }
}

double kschreiber::getIbconst(){
  double wert = getIb()/1023.0;
  return wert;
}

double kschreiber::getIb(){
  double wert=0.0;
  
  if(i10ua->isChecked()){
    wert=10.0;
  }
  if(i100ua->isChecked()){
    wert=100.0;
  }
  if(i1ma->isChecked()){
    wert=1000.0;
  }
  return wert;
}

void kschreiber::boxui1fuellen(){
  QString qs;
  double w;
  int i;
  
  boxui1->clear();
  switch(kurventyp){
  case eausgang:
    switch(halbleitertyp){
      case enpn:
      case epnp:
        if(i10ua->isChecked()){
          w = 10.0;
          for(i=1;i<20;i++){
            qs.sprintf("%2.1f",w);
            boxui1->addItem(qs);
            w = w - 0.5;
          }
        }
        if(i100ua->isChecked()){
          w = 100.0;
          for(i=1;i<20;i++){
            qs.sprintf("%2.0f",w);
            boxui1->addItem(qs);
            w = w - 5.0;
          }
        }
        if(i1ma->isChecked()){
          w = 1000.0;
          for(i=1;i<20;i++){
            qs.sprintf("%2.0f",w);
            boxui1->addItem(qs);
            w = w - 50.0;
          }
        }
      break;
      case enmosfet:
      case epmosfet:
        w = 10.0;
        for(i=1;i<=100;i++){
          qs.sprintf("%2.1f",w);
          boxui1->addItem(qs);
          w = w - 0.1;
        }
      break;
      case enjfet:
      case epjfet:
        w = 10.0;
        for(i=1;i<=101;i++){
          qs.sprintf("%2.1f",w);
          boxui1->addItem(qs);
          w = w - 0.2;
        }
      break;
      case ediode:
        w = 10.0;
        for(i=1;i<=20;i++){
          qs.sprintf("%2.1f",w);
          boxui1->addItem(qs);
          w = w - 0.5;
        }
      break;
    }
    break;
    case eeingang:
      switch(halbleitertyp){
        case enpn:
        case epnp:
        case enmosfet:
        case epmosfet:
        case enjfet:
        case epjfet:
          w = 10.0;
          for(i=1;i<=100;i++){
            qs.sprintf("%2.1f",w);
            boxui1->addItem(qs);
            w = w - 0.1;
          }
        break;
        case ediode:
          w = 10.0;
          for(i=1;i<=20;i++){
            qs.sprintf("%2.1f",w);
            boxui1->addItem(qs);
            w = w - 0.5;
          }
        break;
      }
    break;
  }
}

void kschreiber::boxui2fuellen(){
  
  boxui2->clear();
  switch(halbleitertyp){
    case enpn:
    case epnp:
    case enmosfet:
    case epmosfet:
    case enjfet:
    case epjfet:
      boxui2->addItem("500");
      boxui2->addItem("400");
      boxui2->addItem("300");
      boxui2->addItem("200");
      boxui2->addItem("100");
      boxui2->addItem("90");
      boxui2->addItem("80");
      boxui2->addItem("70");
      boxui2->addItem("60");
      boxui2->addItem("50");
      boxui2->addItem("40");
      boxui2->addItem("30");
      boxui2->addItem("20");
      boxui2->addItem("10");
    break;
    case ediode:
    break;
  }
}

void kschreiber::cfgladen(){
  configfile cfgk;
  int r,g,b,a;
  QColor color;
  
  cfgk.open(".ksch.cfg");
  ucdeinnpn = cfgk.readDouble("ucd_ein_npn", 7.0);
  ucdeinpnp = cfgk.readDouble("ucd_ein_pnp", 7.0);
  ucdeinnmos = cfgk.readDouble("ucd_ein_nmos", 7.0);
  ucdeinpmos = cfgk.readDouble("ucd_ein_pmos", 7.0);
  ucdeinnjfet = cfgk.readDouble("ucd_ein_njfet", 7.0);
  ucdeinpjfet = cfgk.readDouble("ucd_ein_pjfet", 7.0);
  udiode = cfgk.readDouble("u_diode", 1.0);
  ibausnpn = cfgk.readDouble("ib_aus_npn", 200.0);
  ibauspnp = cfgk.readDouble("ib_aus_pnp", 200.0);
  ugausnmos = cfgk.readDouble("ug_aus_nmos", 4.0);
  ugauspmos = cfgk.readDouble("ug_aus_pmos", 4.0);
  ugausnjfet = cfgk.readDouble("ug_aus_njfet", -1.0);
  ugauspjfet = cfgk.readDouble("ug_aus_pjfet", -1.0);
  imaxnpn = cfgk.readDouble("imax_npn", 100.0);
  imaxpnp = cfgk.readDouble("imax_pnp", 100.0);
  imaxnmos = cfgk.readDouble("imax_nmos", 200.0);
  imaxpmos = cfgk.readDouble("imax_pmos", 200.0);
  imaxnjfet = cfgk.readDouble("imax_njfet", 20.0);
  imaxpjfet = cfgk.readDouble("imax_pjfet", 20.0);
  r = cfgk.readInteger("hintergrundr", 255);
  g = cfgk.readInteger("hintergrundg", 255);
  b = cfgk.readInteger("hintergrundb", 152);
  a = cfgk.readInteger("hintergrunda", 255);
  colorhintergrund = QColor(r,g,b,a);
  ksettings.setcolorhintergrund(colorhintergrund);
  //Button einfaerben
  QPalette palette = bcolorhintergrund->palette();
  palette.setColor(QPalette::Active, QPalette::Button, colorhintergrund);
  bcolorhintergrund->setPalette(palette);
  r = cfgk.readInteger("schriftr", 118);
  g = cfgk.readInteger("schriftg", 117);
  b = cfgk.readInteger("schriftb", 136);
  a = cfgk.readInteger("schrifta", 255);
  colorschrift = QColor(r,g,b,a);
  ksettings.setcolorschrift(colorschrift);
  r = cfgk.readInteger("ueberschriftr", 0);
  g = cfgk.readInteger("ueberschriftg", 0);
  b = cfgk.readInteger("ueberschriftb", 0);
  a = cfgk.readInteger("ueberschrifta", 255);
  ueberschriftsize = cfgk.readInteger("ueberschriftsize", 16);
  colorueberschrift = QColor(r,g,b,a);
  ksettings.setcolorueberschrift(colorueberschrift, ueberschriftsize);
  r = cfgk.readInteger("clinier", 255);
  g = cfgk.readInteger("clinieg", 0);
  b = cfgk.readInteger("clinieb", 0);
  a = cfgk.readInteger("cliniea", 255);
  colorlinie = QColor(r,g,b,a);
  ksettings.setcolorlinie(colorlinie);
  r = cfgk.readInteger("chlinier", 0);
  g = cfgk.readInteger("chlinieg", 0);
  b = cfgk.readInteger("chlinieb", 255);
  a = cfgk.readInteger("chliniea", 255);
  colorhlinie = QColor(r,g,b,a);
  ksettings.setcolorhlinie(colorhlinie);

  cfgk.close();
}

void kschreiber::cfgspeichern(){
  configfile cfgk;
  int r,g,b,a;
  QColor color;

  cfgk.open(".ksch.cfg");
  cfgk.writeDouble("ucd_ein_npn", ucdeinnpn);
  cfgk.writeDouble("ucd_ein_pnp", ucdeinpnp);
  cfgk.writeDouble("ucd_ein_nmos", ucdeinnmos);
  cfgk.writeDouble("ucd_ein_pmos", ucdeinpmos);
  cfgk.writeDouble("ucd_ein_njfet", ucdeinnjfet);
  cfgk.writeDouble("ucd_ein_pjfet", ucdeinpjfet);
  cfgk.writeDouble("u_diode", udiode);
  cfgk.writeDouble("ib_aus_npn", ibausnpn);
  cfgk.writeDouble("ib_aus_pnp", ibauspnp);
  cfgk.writeDouble("ug_aus_nmos", ugausnmos);
  cfgk.writeDouble("ug_aus_pmos", ugauspmos);
  cfgk.writeDouble("ug_aus_njfet", ugausnjfet);
  cfgk.writeDouble("ug_aus_pjfet", ugauspjfet);
  cfgk.writeDouble("imax_npn", imaxnpn);
  cfgk.writeDouble("imax_pnp", imaxpnp);
  cfgk.writeDouble("imax_nmos", imaxnmos);
  cfgk.writeDouble("imax_pmos", imaxpmos);
  cfgk.writeDouble("imax_njfet", imaxnjfet);
  cfgk.writeDouble("imax_pjfet", imaxpjfet);
  colorhintergrund.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("hintergrundr", r);
  cfgk.writeInteger("hintergrundg", g);
  cfgk.writeInteger("hintergrundb", b);
  cfgk.writeInteger("hintergrunda", a);
  colorschrift.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("schriftr", r);
  cfgk.writeInteger("schriftg", g);
  cfgk.writeInteger("schriftb", b);
  cfgk.writeInteger("schrifta", a);
  colorueberschrift.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("ueberschriftr", r);
  cfgk.writeInteger("ueberschriftg", g);
  cfgk.writeInteger("ueberschriftb", b);
  cfgk.writeInteger("ueberschrifta", a);
  colorlinie.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("clinier", r);
  cfgk.writeInteger("clinieg", g);
  cfgk.writeInteger("clinieb", b);
  cfgk.writeInteger("cliniea", a);
  colorhlinie.getRgb(&r, &g, &b, &a);
  cfgk.writeInteger("chlinier", r);
  cfgk.writeInteger("chlinieg", g);
  cfgk.writeInteger("chlinieb", b);
  cfgk.writeInteger("chliniea", a);
  cfgk.close();
}

void kschreiber::getUI(){
  double ui1;
  double ui2;
  bool ok;
  QString qs;
  
  qs = boxui1->currentText();
  ui1 = qs.toDouble(&ok);
  qs = boxui2->currentText();
  ui2 = qs.toDouble(&ok);
  
  switch(kurventyp){
    case eeingang:
      switch(halbleitertyp){
        case enpn:
          ucdeinnpn = ui1;
          imaxnpn = ui2;
        break;
        case epnp:
          ucdeinpnp = ui1;
          imaxpnp = ui2;
        break;
        case enmosfet:
          ucdeinnmos = ui1;
          imaxnmos = ui2;
        break;
        case epmosfet:
          ucdeinpmos = ui1;
          imaxpmos = ui2;
        break;
        case enjfet:
          ucdeinnjfet = ui1;
          imaxnjfet = ui2;
        break;
        case epjfet:
          ucdeinpjfet = ui1;
          imaxpjfet = ui2;
        break;
        case ediode:
          udiode = ui1;
        break;
      }
    break;
    case eausgang:
      switch(halbleitertyp){
        case enpn:
          ibausnpn = ui1;
          imaxnpn = ui2;
        break;
        case epnp:
          ibauspnp = ui1;
          imaxpnp = ui2;
        break;
        case enmosfet:
          ugausnmos = ui1;
          imaxnmos = ui2;
        break;
        case epmosfet:
          ugauspmos = ui1;
          imaxpmos = ui2;
        break;
        case enjfet:
          ugausnjfet = ui1;
          imaxnjfet = ui2;
        break;
        case epjfet:
          ugauspjfet = ui1;
          imaxpjfet = ui2;
        break;
        case ediode:
          udiode = ui1;
        break;
      }
    break;
  }
}

void kschreiber::setUI(){
  double ui1=0.0;
  double ui2=0.0;
  bool ok;
  QString qs;
  int auswahl;
  
  switch(kurventyp){
    case eeingang:
      switch(halbleitertyp){
        case enpn:
          ui1 = ucdeinnpn;
          ui2 = imaxnpn;
        break;
        case epnp:
          ui1 = ucdeinpnp;
          ui2 = imaxpnp;
        break;
        case enmosfet:
          ui1 = ucdeinnmos;
          ui2 = imaxnmos;
        break;
        case epmosfet:
          ui1 = ucdeinpmos;
          ui2 = imaxpmos;
        break;
        case enjfet:
          ui1 = ucdeinnjfet;
          ui2 = imaxnjfet;
        break;
        case epjfet:
          ui1 = ucdeinpjfet;
          ui2 = imaxpjfet;
        break;
        case ediode:
          ui1 = udiode;
        break;
      }
    break;
    case eausgang:
      switch(halbleitertyp){
        case enpn:
          ui1 = ibausnpn;
          auswahl = 0;
          if(ui1 > 10.0)auswahl = 1;
          if(ui1 > 100.0)auswahl = 2;
          switch(auswahl){
            case 0:
              i10ua->setChecked(TRUE);
              clickb10uA();
            break;
            case 1:
              i100ua->setChecked(TRUE);
              clickb100uA();
            break;
            case 2:
              i1ma->setChecked(TRUE);
              clickb1mA();
            break;
          }
          ui2 = imaxnpn;
        break;
        case epnp:
          ui1 = ibauspnp;
          i10ua->setChecked(TRUE);
          clickb10uA();
          if(ui1 > 10.0){
            i100ua->setChecked(TRUE);
            clickb100uA();
          }
          if(ui1 > 100.0){
            i1ma->setChecked(TRUE);
            clickb1mA();
          }
          ui2 = imaxpnp;
        break;
        case enmosfet:
          ui1 = ugausnmos;
          ui2 = imaxnmos;
        break;
        case epmosfet:
          ui1 = ugauspmos;
          ui2 = imaxpmos;
        break;
        case enjfet:
          ui1 = ugausnjfet;
          ui2 = imaxnjfet;
        break;
        case epjfet:
          ui1 = ugauspjfet;
          ui2 = imaxpjfet;
        break;
        case ediode:
          ui1 = udiode;
        break;
      }
    break;
  }
  for(int i=0; i < boxui1->count();i++){
    boxui1->setCurrentIndex(i);
    qs = boxui1->itemText(i);
    if(ui1 == qs.toDouble(&ok))break;
  }
  for(int i=0; i < boxui2->count();i++){
    boxui2->setCurrentIndex(i);
    qs = boxui2->itemText(i);
    if(ui2 == qs.toDouble(&ok))break;
  }
}

void kschreiber::clickcolorhintergrund(){
  bool ok;
  QColor color = colorhintergrund;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorhintergrund = color;
    ksettings.setcolorhintergrund(colorhintergrund);
    setKurvenSchreiber();
    QPalette palette = bcolorhintergrund->palette();
    palette.setColor(QPalette::Active, QPalette::Button, color);
    bcolorhintergrund->setPalette(palette);
  }
}

void kschreiber::clickcolorschrift(){
  bool ok;
  QColor color = colorschrift;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorschrift = color;
    ksettings.setcolorschrift(colorschrift);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorueberschrift(){
  bool ok;
  QColor color = colorueberschrift;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorueberschrift = color;
    ksettings.setcolorueberschrift(colorueberschrift,ueberschriftsize);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorlinie(){
  bool ok;
  QColor color = colorlinie;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorlinie = color;
    ksettings.setcolorlinie(colorlinie);
    setKurvenSchreiber();
  }
}

void kschreiber::clickcolorhlinie(){
  bool ok;
  QColor color = colorhlinie;
  QRgb rgb;
  rgb = color.rgb();
  color = QColorDialog::getRgba(rgb, &ok, this);
  if(ok){
    colorhlinie = color;
    ksettings.setcolorhlinie(colorhlinie);
    setKurvenSchreiber();
  }
}

//QString kschreiber::punkt2komma(QString &qs){
//}

void kschreiber::fopentty(){
  int gelesen;
  
  if(bvantwort){
    geraet = ' ';
    kversion = 0;
    bvantwort = FALSE;
    gelesen = picmodem->readttybuffer(rxbuffer, 2);
    if(gelesen == 2){
      geraet = rxbuffer[0];
      kversion = rxbuffer[1];
      bvantwort = FALSE;
      //qDebug("gelesen: %i",gelesen);
      if((geraet == 'K')and(kversion > 100)){
        bksgefunden = TRUE;
        openttytimer->stop(); //Suchen der Schnittstelle bendet
        statusBar()->showMessage(tr("HW gefunden an: ") + srs232);
        setAText(srs232);
        awidget->setEnabled(TRUE);
        bwstart = TRUE;
      }else{
        idtty++;
        if(idtty > (listtty.count()-1))idtty = 0;
        openttytimer->start(200); //Suchen der Schnittstelle
        picmodem->closetty();
        rs232isopen = FALSE;
      }
    }else{
      idtty++;
      if(idtty > (listtty.count()-1))idtty = 0;
      openttytimer->start(200); //Suchen der Schnittstelle
      picmodem->closetty();
      rs232isopen = FALSE;
    }
  }else{
    if(!rs232isopen){
      srs232 = listtty.at(idtty);
      statusBar()->showMessage("Suche HW :" + srs232);
      //qDebug(qPrintable(srs232));
      //qDebug("%i",idtty);
      rs232isopen = picmodem->opentty(srs232);
      if(rs232isopen){
        defaultlesentty();
        picmodem->writeLine("@v");
        bvantwort = TRUE;
        openttytimer->start(200); //Suchen der Schnittstelle
      }else{
        idtty++;
        if(idtty > (listtty.count()-1))idtty = 0;
        openttytimer->start(100); //Suchen der Schnittstelle
        bvantwort = FALSE;
        picmodem->closetty();
      }
    }
  }
}

void kschreiber::warmstart(){
  if(bwstart){
    QMessageBox::critical(this, tr("Hardware nicht gefunden","WarnungsBox"), 
                                tr("<B>Die USB-Schnittstelle ist nicht zu finden!</B><BR>"
                                   "Die automatische Suche wird aktiviert!","WarnungsBox"));
    idtty=0;
    testtimer->stop();
    imaxtimer->stop();
    akurventimer->stop();
    ekurventimer->stop();
    repainttimer->stop();
    openttytimer->stop();
    picmodem->closetty();
    rs232isopen = FALSE;
    awidget->setEnabled(FALSE);
    kversion = 0;
    setAText("");
    openttytimer->start(200); //Suchen der Schnittstelle
  }
}

